from mcp import types
from mcp.server.fastmcp import FastMCP
from pathlib import Path

cwd = Path.home() / 'Desktop'

mcp = FastMCP("filesystem")

@mcp.tool()
async def get_cwd() -> types.TextContent:
    """Get the current working directory."""
    return types.TextContent(type='text', text=str(cwd))

@mcp.tool()
async def list_cwd() -> types.TextContent:
    """Get the files from the current working directory."""
    return types.TextContent(type='text', text=str(list(cwd.glob('*'))))

@mcp.tool()
async def search_for_file(file_name: str) -> types.TextContent:
    """Search recursively for a file starting from the current working directory
    given the exact name of the file.
    The file format of the file must be provided.
    """
    return types.TextContent(type='text', text=str(list(cwd.rglob(f'*{file_name}'))))

@mcp.tool()
async def search_for_file_partial(file_or_folder_name: str) -> types.TextContent:
    """Search recursively for a file or folder starting from the current working directory.
    """
    return types.TextContent(type='text', text=str(list(cwd.rglob(f'*{file_or_folder_name}*'))))

@mcp.tool()
async def make_desktop_cwd() -> types.TextContent:
    """Make the Desktop the current working directory."""
    global cwd
    cwd = Path.home() / 'Desktop'
    return types.TextContent(type='text', text='Done')

@mcp.tool()
async def make_folder_cwd(folder_name: str) -> types.TextContent:
    """Make a folder the current working directory.
    That folder must be present in the previous cwd.
    """
    global cwd
    new_cwd = cwd / folder_name
    if new_cwd.exists():
        result = 'Done'
        cwd = new_cwd
    else:
        result = f'Folder not found. The previous cwd is maintained: {cwd}'
    return types.TextContent(type='text', text=result)


def main():
    mcp.run(transport='stdio')


if __name__ == '__main__':
    main()


import sys
from pathlib import Path
from environment import Environment

import requests

ADFIN_URL = Environment.get('ADFIN_URL', 'https://app.adfin.com')


class AdfinSession:
    def __init__(self):
        self.adfin_username = Environment.get('ADFIN_EMAIL')
        self.adfin_password = Environment.get('ADFIN_PASSWORD')

        self.access_token = None
        self.access_token = self.login()['accessToken']

    def _get_headers(self):
        if self.access_token is None:
            return None
        else:
            return {'Authorization': f'Bearer {self.access_token}'}

    def call_route(self, method_type, route, body: dict = None):
        method = getattr(requests, method_type)
        if not route.startswith('/'):
            route = '/' + route

        if method_type == 'get':
            response = method(f"{ADFIN_URL}/api{route}", params=body, headers=self._get_headers())
        else:
            response = method(f"{ADFIN_URL}/api{route}", json=body, headers=self._get_headers())

        if response.status_code == 200:
            return response.json()
        else:
            message = f"Error: {response.status_code} {response.text}"
            print(message, file=sys.stderr)

            print('Will attempt to refresh the access token', file=sys.stderr)
            self.access_token = self.login()['accessToken']
            print('Refreshed the access token', file=sys.stderr)

            response = method(f"{ADFIN_URL}/api{route}", json=body, headers=self._get_headers())
            if response.status_code == 200:
                return response.json()
            else:
                message = f"Error: {response.status_code} {response.text}"
                print(message, file=sys.stderr)
                return message

    def login(self) -> dict:
        return self.call_route('post', '/auth/login',
                        body={'username': self.adfin_username, 'password': self.adfin_password})

    def get_invoices(self):
        return self.call_route('get', 'invoices')

    def upload_invoice(self, invoice_path):
        UPLOAD_URL = ADFIN_URL + "/api/process:invoiceUpload"
        headers = {"Authorization": f"Bearer {self.access_token}"}

        with open(invoice_path, "rb") as file:
            files = {"invoice": (Path(invoice_path).name, file, "application/pdf")}
            response = requests.post(UPLOAD_URL, headers=headers, files=files)

        if response.status_code == 200:
            message = 'Upload successful'
            print(message, file=sys.stderr)
            return message
        else:
            message = f"Error: {response.status_code} {response.text}"
            print(message, file=sys.stderr)
            return f'Error: Could not upload invoice - {message}'


if __name__ == '__main__':
    adfin = AdfinSession()

    access_token = adfin.access_token
    print("Access token:", access_token)




import sys

import requests


def download_api_documentation(url='https://app.adfin.com/api/docs/Public-APIs'):
    response = requests.get(url)
    if response.status_code == 200:
        return response.json(), True
    else:
        message = f"Error: {response.status_code} {response.text}"
        print(message, file=sys.stderr)
        return {'code': response.status_code, 'message': message}, False


def load_apis():
    loaded, success = download_api_documentation()
    if not success:
        raise Exception("Failed to download API documentation")
    paths = loaded['paths']
    components = loaded['components']['schemas']
    method_data = []
    for path, methods in paths.items():
        for method_name, method in methods.items():
            summary = method.get('summary')  # might be null
            success_code = '200'
            if success_code not in method['responses']:
                success_code = [e for e in method['responses'] if e.startswith('2')]
                if len(success_code) == 0:
                    success_code = [e for e in method['responses'] if e.startswith('3')][0]
                else:
                    success_code = success_code[0]
            response_description = method['responses'][success_code]['description']
            request_body = method.get('requestBody')
            request_data = None

            if request_body:
                content = request_body['content']
                data_type = list(content.keys())[0]
                schema = content[data_type]['schema']
                properties = schema.get('properties')
                if properties:
                    request_data = {key: value.get('description') for key, value in properties.items()}
                else:
                    if 'type' in schema and 'description' in schema:
                        request_data = {schema['type']: schema['description']}
                    else:
                        for key, value in schema.items():
                            if key == '$ref':
                                value = value.split('/')[-1]
                                request_data = components[value]['properties']
                            else:
                                raise ValueError(f"Unknown key: {key}")

            # Parameters parsing (query/path)
            request_params = method.get('parameters', [])
            parameters_data = []
            for param in request_params:
                param_schema = param.get('schema', {})
                parameters_data.append({
                    'name': param.get('name', 'Unnamed'),
                    'location': param.get('in', 'unknown'),  # "query" or "path"
                    'required': param.get('required', False),
                    'type': param_schema.get('type', 'unknown'),
                    'description': param.get('description', 'No description provided')
                })

            method_data.append({
                'route': path,
                'method': method_name,
                'summary': summary,
                'response_description': response_description,
                'request_data': request_data,
                'parameters': parameters_data
            })

    return method_data


if __name__ == '__main__':
    load_apis()



import os

from dotenv import load_dotenv

load_dotenv()

def get_sensitive_data():
    return {
        'adfin email': Environment.get('ADFIN_EMAIL'),
        'adfin password': Environment.get('ADFIN_PASSWORD'),
        'actionstep email': Environment.get('ACTIONSTEP_USERNAME'),
        'actionstep password': Environment.get('ACTIONSTEP_PASSWORD'),
    }

class Environment:
    @staticmethod
    def get(__key, __default=None):
        return os.getenv(__key, __default)

    __getattr__ = dict.get



from mcp import types
from mcp.server.fastmcp import FastMCP
from adfin_interaction import AdfinSession
from api_importer import load_apis
import sys

adfin = AdfinSession()
mcp = FastMCP("adfin")

@mcp.tool()
def upload_invoice_to_adfin(invoice_path: str) -> types.TextContent:
    """Upload an invoice to Adfin given its absolute path."""
    response = adfin.upload_invoice(invoice_path)
    return types.TextContent(type='text', text=response)

def register_all_apis():
    apis = load_apis()
    for api in apis:
        response_description = api["response_description"]

        clean_route = (api['route'].replace('/', '_').replace('-', '_').replace(':', '_')
                       .replace('{', '').replace('}', ''))

        if 'invoiceUpload' in clean_route:
            continue

        method_name = f'{api['method']}{clean_route}'
        clean_summary = api["summary"]
        if clean_summary is None:
            words = method_name.split('_')
            words[0] = words[0].capitalize()
            clean_summary = ' '.join(words) + '. '

        doc = clean_summary + '. '
        if response_description != "OK":
            doc += f'If successful, the outcome will be \"{api["response_description"]}\". '
        if api['request_data']:
            doc += f'The request data should be: {api["request_data"]}.'
        if api['parameters']:
            doc += f'The parameters should be: {api["parameters"]}.'
        else:
            doc += f'The request data should be an empty dictionary {{}}.'

# Claude is inconsistent in the way it provides a request_data that should be empty.
        method_str = f"""
@mcp.tool()
def {method_name}(request_data: dict = None) -> types.TextContent:
    \"\"\"{doc}\"\"\"
    print("The input of method {method_name} is", request_data, file=sys.stderr)
    if request_data is None or request_data == "" or request_data == "null" or request_data.get("request_data") == "null":
        request_data = None
    route = \"{api['route']}\"
    if '{{' in route:
        route = route.format(**request_data)
    response = adfin.call_route(\"{api['method']}\", route, request_data)
    print("The input of method {method_name} is", response, file=sys.stderr)
    return types.TextContent(type='text', text=str(response))
"""
        exec(method_str)

register_all_apis()


def main():
    mcp.run(transport='stdio')


if __name__ == "__main__":
    main()